"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _regenerator = _interopRequireDefault(require("@babel/runtime/regenerator"));

var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _pixelmatch = _interopRequireDefault(require("pixelmatch"));

var _pngjs = require("pngjs");

var _utils = require("./utils");

var _config = _interopRequireDefault(require("./config"));

var setupFolders = function setupFolders() {
  (0, _utils.createDir)([_config["default"].dir.baseline, _config["default"].dir.comparison, _config["default"].dir.diff]);
};

var copyScreenshot = function copyScreenshot(args) {
  // If baseline does not exist, copy comparison image to baseline folder
  if (!_fsExtra["default"].existsSync(_config["default"].image.baseline(args.testName))) {
    _fsExtra["default"].copyFileSync(_config["default"].image.comparison(args.testName), _config["default"].image.baseline(args.testName));
  }

  return true;
};

function compareSnapshotsPlugin(_x) {
  return _compareSnapshotsPlugin.apply(this, arguments);
}

function _compareSnapshotsPlugin() {
  _compareSnapshotsPlugin = (0, _asyncToGenerator2["default"])( /*#__PURE__*/_regenerator["default"].mark(function _callee(args) {
    var baselineImg, comparisonImg, diff, comparisonFullCanvas, baselineFullCanvas, pixelMismatchResult, percentage;
    return _regenerator["default"].wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return (0, _utils.parseImage)(_config["default"].image.baseline(args.testName));

          case 2:
            baselineImg = _context.sent;
            _context.next = 5;
            return (0, _utils.parseImage)(_config["default"].image.comparison(args.testName));

          case 5:
            comparisonImg = _context.sent;
            diff = new _pngjs.PNG({
              width: Math.max(comparisonImg.width, baselineImg.width),
              height: Math.max(comparisonImg.height, baselineImg.height)
            });
            _context.next = 9;
            return (0, _utils.adjustCanvas)(comparisonImg, diff.width, diff.height);

          case 9:
            comparisonFullCanvas = _context.sent;
            _context.next = 12;
            return (0, _utils.adjustCanvas)(baselineImg, diff.width, diff.height);

          case 12:
            baselineFullCanvas = _context.sent;
            pixelMismatchResult = (0, _pixelmatch["default"])(baselineFullCanvas.data, comparisonFullCanvas.data, diff.data, baselineFullCanvas.width, baselineFullCanvas.height, {
              threshold: 0.1
            });
            percentage = Math.pow(pixelMismatchResult / diff.width / diff.height, 0.5);

            if (percentage > args.threshold) {
              diff.pack().pipe(_fsExtra["default"].createWriteStream(_config["default"].image.diff(args.testName)));
            }

            return _context.abrupt("return", percentage);

          case 17:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));
  return _compareSnapshotsPlugin.apply(this, arguments);
}

var getCompareSnapshotsPlugin = function getCompareSnapshotsPlugin(on) {
  setupFolders();
  on('after:screenshot', function (details) {
    // We rename cypress screenshot and move it to our own
    // managed folder structure that lives on ./config.js
    return new Promise(function (resolve, reject) {
      _fsExtra["default"].rename(details.path, _config["default"].image.comparison(details.name), function (err) {
        if (err) return reject(err);
        resolve({
          path: _config["default"].image.comparison(details.name)
        });
      });
    });
  });
  on('task', {
    compareSnapshotsPlugin: compareSnapshotsPlugin,
    copyScreenshot: copyScreenshot
  });
};

module.exports = getCompareSnapshotsPlugin;